#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using FormationGamePrototype.Graphics;
using FormationGamePrototype.World;
using FormationGamePrototype.Helper;
#endregion

namespace FormationGamePrototype
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        const uint PLAYER_MODEL_COUNT = 6;

        GraphicsDeviceManager graphics;
        ContentManager content;
        //SteeringObject[] playerModels;

        PlayerShip playerModel;
        List<SteeringObject> modelList;

        bool AddKeyPressed;
        bool DeleteKeyPressed;
        bool F1KeyPressed;
        bool F2KeyPressed;

        // The aspect ratio determines how to scale 3d to 2d projection.
        float aspectRatio;

        public Game1()
        {
            this.IsFixedTimeStep = false;

            AddKeyPressed = false;
            DeleteKeyPressed = false;
            F1KeyPressed = false;
            F2KeyPressed = false;

            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);

            modelList = new List<SteeringObject>(16);

            playerModel = new PlayerShip(this);
            this.Components.Add(playerModel);
            modelList.Add(playerModel);
            /*playerModels = new SteeringObject[PLAYER_MODEL_COUNT];
            for( uint i = 0; i < PLAYER_MODEL_COUNT; ++i )
            {
                SimpleModel model = new SimpleModel(this);
                this.Components.Add(model);

                if( i > 0 )
                    playerModels[i] = new FollowerShip(this, model, (PlayerShip)playerModels[0]);
                else
                    playerModels[i] = new PlayerShip(this, model);
                this.Components.Add(playerModels[i]);
            }*/
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }



        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                DebugText.LoadResources(graphics, content);
                DebugLine.LoadResources(graphics);

                playerModel.Model.LoadModel(content, "Content\\Models\\p1_wedge");
                playerModel.Position = new Vector2(-1500.0f, 1750.0f);
                playerModel.Model.Rotation = -MathHelper.PiOver2;

                /*Random rand = new Random();
                for (uint i = 0; i < PLAYER_MODEL_COUNT; ++i)
                {
                    if (i == 0)
                    {
                        playerModels[0].Model.LoadModel(content, "Content\\Models\\p1_wedge");
                        playerModels[0].Position = new Vector2(-1500.0f, 1750.0f);
                    }
                    else
                    {
                        int charNum = rand.Next(3);
                        if( charNum == 0 )
                            playerModels[i].Model.LoadModel(content, "Content\\Models\\p2_wedge");
                        else if( charNum == 1 )
                            playerModels[i].Model.LoadModel(content, "Content\\Models\\p2_saucer");
                        else if( charNum == 2 )
                            playerModels[i].Model.LoadModel(content, "Content\\Models\\p1_saucer");

                        ((PlayerShip)playerModels[0]).AddFollower(PLAYER_MODEL_COUNT - 1, (FollowerShip)playerModels[i]);
                    }
                }
                
                for (uint i = 0; i < PLAYER_MODEL_COUNT; ++i)
                    playerModels[i].Model.Rotation = -MathHelper.PiOver2;*/
            }

            // TODO: Load any ResourceManagementMode.Manual content
            aspectRatio = graphics.GraphicsDevice.Viewport.Width / graphics.GraphicsDevice.Viewport.Height;

        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent)
            {
                // TODO: Unload any ResourceManagementMode.Automatic content
                content.Unload();
            }

            // TODO: Unload any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            DebugText.Output("Formation Prototype(Press ESC to exit)");
            DebugText.Output("Press F1 to toggle debut text.  Press F2 to toggle debug lines.");
            DebugText.Output("Move via arrow keys. Press A to add ship. Press B to remove ship.");
            DebugText.Output("Follower Count: {0} Max: {1}", modelList.Count - 1, playerModel.MaximumFollowerCount);

            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();

            if (Keyboard.GetState().IsKeyDown(Keys.F1) && !F1KeyPressed)
            {
                DebugText.Visible = !DebugText.Visible;
                F1KeyPressed = true;
            }
            else if (Keyboard.GetState().IsKeyUp(Keys.F1) && F1KeyPressed)
            {
                F1KeyPressed = false;
            }

            if (Keyboard.GetState().IsKeyDown(Keys.F2) && !F2KeyPressed)
            {
                DebugLine.Visible = !DebugLine.Visible;
                F2KeyPressed = true;
            }
            else if (Keyboard.GetState().IsKeyUp(Keys.F2) && F2KeyPressed)
            {
                F2KeyPressed = false;
            }

            if (Keyboard.GetState().IsKeyDown(Keys.A) && !AddKeyPressed)
            {
                AddKeyPressed = true;
                AddFollower();
            }
            else if( Keyboard.GetState().IsKeyUp(Keys.A) && AddKeyPressed )
            {
                AddKeyPressed = false;
            }

            if (Keyboard.GetState().IsKeyDown(Keys.D) && !DeleteKeyPressed)
            {
                DeleteKeyPressed = true;
                DeleteFollower();
            }
            else if( Keyboard.GetState().IsKeyUp(Keys.D) && DeleteKeyPressed )
            {
                DeleteKeyPressed = false;
            }


            base.Update(gameTime);
        }

        public void AddFollower()
        {
            FollowerShip newFollower = new FollowerShip(this, playerModel);
            Random rand = new Random();
            int charNum = rand.Next(3);
            if( charNum == 0 )
                newFollower.Model.LoadModel(content, "Content\\Models\\p2_wedge");
            else if( charNum == 1 )
                newFollower.Model.LoadModel(content, "Content\\Models\\p2_saucer");
            else if( charNum == 2 )
                newFollower.Model.LoadModel(content, "Content\\Models\\p1_saucer");
            newFollower.Model.Rotation = -MathHelper.PiOver2;

            if (playerModel.PushFollower(newFollower))
            {
                this.Components.Add(newFollower);
                modelList.Add(newFollower);
            }
        }

        public void DeleteFollower()
        {
            FollowerShip oldFollower = playerModel.PopFollower();
            if (oldFollower != null)
            {
                this.Components.Remove(oldFollower);
                modelList.Remove(oldFollower);
                oldFollower.Destroy();
            }
        }

        // Set the position of the camera in world space, for our view matrix.
        Vector3 cameraPosition = new Vector3(0.0f, 0.0f, 30000.0f);

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            Matrix myView = Matrix.CreateLookAt(cameraPosition, Vector3.Zero, Vector3.Up);
            Matrix myProjection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45.0f), aspectRatio, 1.0f, 50000.0f);

            /*for( uint i = 0; i < PLAYER_MODEL_COUNT; ++i )
            {
                playerModels[i].Model.SetViewMatrix(myView);
                playerModels[i].Model.SetProjectionMatrix(myProjection);
            }*/
            foreach( SteeringObject mdl in modelList )
            {
                mdl.Model.SetViewMatrix(myView);
                mdl.Model.SetProjectionMatrix(myProjection);
            }

            DebugLine.SetViewMatrix(myView);
            DebugLine.SetProjectionMatrix(myProjection);
            DebugLine.DrawLines(graphics);
            
            DebugText.DrawStrings();
            base.Draw(gameTime);
        }
    }
}
