#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace FormationGamePrototype.Graphics
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class SimpleModel : Microsoft.Xna.Framework.DrawableGameComponent
    {
        // Set the 3D model to draw.
        Model myModel;
        Matrix viewMatrix;
        Matrix projectionMatrix;

        // Set the position of the model in world space, and set the rotation.
        Vector3 modelPosition = Vector3.Zero;
        float modelRotation = 0.0f;

        public SimpleModel(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here
            
            base.Initialize();
        }

        public void LoadModel(ContentManager content, String modelPath)
        {
            // TODO: Add your initialization code here
            //myModel = content.Load<Model>("Content\\Models\\p1_wedge");
            myModel = content.Load<Model>(modelPath);
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here

            base.Update(gameTime);
        }

        public Vector3 Position
        {
            set
            {
                modelPosition = value;
            }
            get
            {
                return modelPosition;
            }
        }

        public float Rotation
        {
            set
            {
                modelRotation = value;
            }
            get
            {
                return modelRotation;
            }
        }

        public void SetViewMatrix(Matrix view)
        {
            viewMatrix = view;
        }

        public void SetProjectionMatrix(Matrix projection)
        {
            projectionMatrix = projection;
        }

        public override void Draw(GameTime gameTime)
        //public override void Draw(GameTime gameTime, Matrix viewMatrix, Matrix projectionMatrix)
        {
            // Copy any parent transforms.
            Matrix[] transforms = new Matrix[myModel.Bones.Count];
            myModel.CopyAbsoluteBoneTransformsTo(transforms);

            // Draw the model. A model can have multiple meshes, so loop.
            foreach (ModelMesh mesh in myModel.Meshes)
            {
                // This is where the mesh orientation is set, as well as our camera and projection.
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.EnableDefaultLighting();
                    effect.World = transforms[mesh.ParentBone.Index] * Matrix.CreateRotationY(modelRotation)
                        * Matrix.CreateTranslation(modelPosition);
                    effect.View = viewMatrix;
                    effect.Projection = projectionMatrix;
                    //effect.View = Matrix.CreateLookAt(cameraPosition, Vector3.Zero, Vector3.Up);
                    //effect.Projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45.0f), aspectRatio, 1.0f, 10000.0f);
                }
                // Draw the mesh, using the effects set above.
                mesh.Draw();
            }

            base.Update(gameTime);
        }
    }
}


