using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace FormationGamePrototype.Helper
{
    public static class DebugLine
    {
        class LineData
        {
            public Vector2 p1;
            public Vector2 p2;
            public Color color;

            public LineData(Vector2 _p1, Vector2 _p2, Color _color)
            {
                p1 = _p1;
                p2 = _p2;
                color = _color;
            }
        }

        static bool m_bVisible;

        static List<LineData> m_LineList;

        static BasicEffect m_BasicEffect;
        static VertexDeclaration m_VertexDeclaration;

        static VertexPositionColor[] m_PointList;
        static VertexBuffer m_VertexBuffer;

        static short[] m_LineListIndices;
        

        static DebugLine()
        {
            m_bVisible = true;
            m_LineList = new List<LineData>(64);
        }

        static public bool Visible
        {
            get
            {
                return m_bVisible;
            }
            set
            {
                m_bVisible = value;
            }
        }

        static public void LoadResources(GraphicsDeviceManager graphics)
        {
            m_VertexDeclaration = new VertexDeclaration(
            graphics.GraphicsDevice,
            VertexPositionColor.VertexElements);

            m_BasicEffect = new BasicEffect(graphics.GraphicsDevice, null);
            m_BasicEffect.DiffuseColor = new Vector3(1.0f, 1.0f, 1.0f);

            m_BasicEffect.View = Matrix.Identity;
            m_BasicEffect.Projection = Matrix.Identity;
            m_BasicEffect.VertexColorEnabled = true;
        }

        static public void SetViewMatrix(Matrix viewMatrix)
        {
            m_BasicEffect.View = viewMatrix;
        }

        static public void SetProjectionMatrix(Matrix projectionMatrix)
        {
            m_BasicEffect.Projection = projectionMatrix;
        }

        static public void Draw(Vector2 p1, Vector2 p2, Color lineColor)
        {
            m_LineList.Add(new LineData(p1, p2, lineColor));
        }

        static public void DrawLines(GraphicsDeviceManager graphics)
        {
            if (m_bVisible)
            {
                graphics.GraphicsDevice.VertexDeclaration = m_VertexDeclaration;

                if (m_LineList.Count > 0)
                {
                    m_PointList = new VertexPositionColor[m_LineList.Count * 2];
                    int currentPoint = 0;
                    for (int i = 0; i < m_LineList.Count; ++i)
                    {
                        m_PointList[currentPoint] = new VertexPositionColor(new Vector3(m_LineList[i].p1, 0.0f), m_LineList[i].color);
                        ++currentPoint;
                        m_PointList[currentPoint] = new VertexPositionColor(new Vector3(m_LineList[i].p2, 0.0f), m_LineList[i].color);
                        ++currentPoint;
                    }

                    // Initialize the vertex buffer, allocating memory for each vertex.
                    m_VertexBuffer = new VertexBuffer(graphics.GraphicsDevice,
                        VertexPositionColor.SizeInBytes * (m_PointList.Length),
                        ResourceUsage.None,
                        ResourceManagementMode.Automatic);

                    // Set the vertex buffer data to the array of vertices.
                    m_VertexBuffer.SetData<VertexPositionColor>(m_PointList);



                    // Initialize an array of indices of type short.
                    m_LineListIndices = new short[m_LineList.Count * 2];

                    // Populate the array with references to indices in the vertex buffer
                    for (int i = 0; i < m_LineList.Count * 2; ++i)
                    {
                        m_LineListIndices[i] = (short)i;
                    }


                    // The effect is a compiled effect created and compiled elsewhere
                    // in the application.
                    m_BasicEffect.Begin();
                    foreach (EffectPass pass in m_BasicEffect.CurrentTechnique.Passes)
                    {
                        pass.Begin();

                        graphics.GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(
                            PrimitiveType.LineList,
                            m_PointList,
                            0,  // vertex buffer offset to add to each element of the index buffer
                            m_LineList.Count * 2,  // number of vertices in pointList
                            m_LineListIndices,  // the index buffer
                            0,  // first index element to read
                            m_LineList.Count   // number of primitives to draw
                        );


                        pass.End();
                    }
                    m_BasicEffect.End();

                }
            }

            m_LineList.Clear();
        }
    }
}
