#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using FormationGamePrototype.Graphics;
#endregion

namespace FormationGamePrototype.World
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class FollowerShip : SteeringObject
    {
        protected PlayerShip m_Parent;
        protected Vector2 m_DockPoint;
        protected Vector2 m_LastSteeringForce;
        Random randomNum;

        public FollowerShip(Game game, PlayerShip ship)
            : base(game)
        {
            m_Parent = ship;
            m_DockPoint = Vector2.Zero;
            randomNum = null;
        }

        public PlayerShip Parent
        {
            get
            {
                return m_Parent;
            }
        }

        public Vector2 DockPoint
        {
            set
            {
                m_DockPoint = value;
            }
            get
            {
                return m_DockPoint;
            }
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            if( randomNum == null )
                randomNum = new Random((int)gameTime.TotalRealTime.Ticks);
            /*Vector2 currentPos = this.Position;
            Vector2 targetPos = m_Parent.Position + m_DockPoint;

            currentPos = currentPos + (targetPos - currentPos) * 0.5f;

            this.Position = currentPos;*/
            Vector2 currentPos = this.Position;
            Vector2 targetPos = m_Parent.Position + m_DockPoint + m_Parent.Velocity;

            Vector2 steeringForce = Vector2.Zero;

            if (Math.Abs(Vector2.Distance(m_Parent.Position, currentPos)) < 2000.0f)
            {
                steeringForce = (m_Parent.Position - currentPos);
                steeringForce.Normalize();
                steeringForce = steeringForce * (m_MaxForce);
                steeringForce = -1.0f * (steeringForce - m_Velocity);
            }
            else if (Math.Abs(Vector2.Distance(targetPos, currentPos)) > 1000.0f)
            {
                /*steeringForce = (targetPos - currentPos);
                steeringForce.Normalize();
                steeringForce = steeringForce * ( m_MaxForce );
                steeringForce = steeringForce - m_Velocity;*/

                Vector2 targetOffset = targetPos - currentPos;
                float distance = targetOffset.Length();
                float rampedSpeed = m_MaxForce * (distance / 1000.0f);
                float clippedSpeed = rampedSpeed;
                if (clippedSpeed > m_MaxForce)
                    clippedSpeed = m_MaxForce;
                Vector2 desiredVelocity = (clippedSpeed / distance) * targetOffset;
                steeringForce = desiredVelocity - m_Velocity;
            }
            else
            {
                //Random rand = new Random((int)DateTime.Now.Ticks);
                //steeringForce.X = ((float)rand.NextDouble() * (m_MaxForce * 0.3f) - (m_MaxForce* 0.15f) );
                //steeringForce.Y = ((float)rand.NextDouble() * (m_MaxForce * 0.3f) - (m_MaxForce * 0.15f));

                int degree = randomNum.Next(15) - (int)(15 * 0.5f);
                float radian = ((float)degree * (float)Math.PI) / 180.0f;

                float lastRotation = (float)Math.Atan2(m_LastSteeringForce.Y, m_LastSteeringForce.X);
                radian += lastRotation;

                steeringForce = new Vector2((m_MaxForce * 0.025f) * (float)Math.Cos(radian), (m_MaxForce * 0.025f) * (float)Math.Sin(radian));
            }

            base.ApplySteeringForce( steeringForce );
            m_LastSteeringForce = steeringForce;

            base.Update(gameTime);
        }
    }
}


