#region Using Statements
using System;
using System.Collections.Generic;
using System.Collections;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using FormationGamePrototype.Graphics;
using FormationGamePrototype.Helper;
#endregion

namespace FormationGamePrototype.World
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class PlayerShip : SteeringObject
    {
        protected ArrayList[] m_DockingPoints;
        List<FollowerShip> m_Followers;

        const int DOCKING_FORMATION_COUNT = 9;

        public PlayerShip(Game game)
            : base(game)
        {
            m_Followers = new List<FollowerShip>(DOCKING_FORMATION_COUNT);
        }

        public int MaximumFollowerCount
        {
            get
            {
                return DOCKING_FORMATION_COUNT;
            }
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            m_DockingPoints = new ArrayList[DOCKING_FORMATION_COUNT];
            for (int i = 0; i < DOCKING_FORMATION_COUNT; ++i)
                m_DockingPoints[i] = new ArrayList(i + 1);
            
            m_DockingPoints[0].Add(new Vector2(3500.0f, 0.0f));

            m_DockingPoints[1].Add(new Vector2(-1500.0f, 2000.0f));
            m_DockingPoints[1].Add(new Vector2(-1500.0f, -2000.0f));

            m_DockingPoints[2].Add(new Vector2(3500.0f, 0.0f));
            m_DockingPoints[2].Add(new Vector2(-1500.0f, 2000.0f));
            m_DockingPoints[2].Add(new Vector2(-1500.0f, -2000.0f));

            m_DockingPoints[3].Add(new Vector2(2000.0f, -2000.0f));
            m_DockingPoints[3].Add(new Vector2(2000.0f, 2000.0f));
            m_DockingPoints[3].Add(new Vector2(-2000.0f, 2000.0f));
            m_DockingPoints[3].Add(new Vector2(-2000.0f, -2000.0f));

            m_DockingPoints[4].Add(new Vector2(2000.0f, -2000.0f));
            m_DockingPoints[4].Add(new Vector2(2500.0f, 0.0f));
            m_DockingPoints[4].Add(new Vector2(2000.0f, 2000.0f));
            m_DockingPoints[4].Add(new Vector2(-2000.0f, 2000.0f));
            m_DockingPoints[4].Add(new Vector2(-2000.0f, -2000.0f));

            m_DockingPoints[5].Add(new Vector2(2000.0f, -3000.0f));
            m_DockingPoints[5].Add(new Vector2(2500.0f, -1000.0f));
            m_DockingPoints[5].Add(new Vector2(2500.0f, 1000.0f));
            m_DockingPoints[5].Add(new Vector2(2000.0f, 3000.0f));
            m_DockingPoints[5].Add(new Vector2(-2000.0f, 2000.0f));
            m_DockingPoints[5].Add(new Vector2(-2000.0f, -2000.0f));

            m_DockingPoints[6].Add(new Vector2(2000.0f, -2500.0f));
            m_DockingPoints[6].Add(new Vector2(2500.0f, 0.0f));
            m_DockingPoints[6].Add(new Vector2(2000.0f, 2500.0f));
            m_DockingPoints[6].Add(new Vector2(-2000.0f, 1500.0f));
            m_DockingPoints[6].Add(new Vector2(-2000.0f, -1500.0f));
            m_DockingPoints[6].Add(new Vector2(-3000.0f, 3500.0f));
            m_DockingPoints[6].Add(new Vector2(-3000.0f, -3500.0f));

            m_DockingPoints[7].Add(new Vector2(-4500.0f, 6500.0f));
            m_DockingPoints[7].Add(new Vector2(-3000.0f, 5000.0f));
            m_DockingPoints[7].Add(new Vector2(-1500.0f, 3500.0f));
            m_DockingPoints[7].Add(new Vector2(0.0f, 2000.0f));
            m_DockingPoints[7].Add(new Vector2(0.0f, -2000.0f));
            m_DockingPoints[7].Add(new Vector2(-1500.0f, -3500.0f));
            m_DockingPoints[7].Add(new Vector2(-3000.0f, -5000.0f));
            m_DockingPoints[7].Add(new Vector2(-4500.0f, -6500.0f));

            m_DockingPoints[8].Add(new Vector2(2500.0f, 2500.0f));
            m_DockingPoints[8].Add(new Vector2(4500.0f, 0.0f));
            m_DockingPoints[8].Add(new Vector2(2500.0f, -2500.0f));
            m_DockingPoints[8].Add(new Vector2(-3500.0f, 5500.0f));
            m_DockingPoints[8].Add(new Vector2(-1500.0f, 4000.0f));
            m_DockingPoints[8].Add(new Vector2(-3500.0f, 2500.0f));
            m_DockingPoints[8].Add(new Vector2(-3500.0f, -2500.0f));
            m_DockingPoints[8].Add(new Vector2(-1500.0f, -4000.0f));
            m_DockingPoints[8].Add(new Vector2(-3500.0f, -5500.0f));

            base.Initialize();
        }

        protected bool AddFollower(int followerIndex, int totalFollowers, FollowerShip follower)
        {
            if (followerIndex > DOCKING_FORMATION_COUNT - 1)
                return false;

            int formationIndex = totalFollowers - 1;
            if (formationIndex > DOCKING_FORMATION_COUNT - 1)
                formationIndex = DOCKING_FORMATION_COUNT - 1;

            follower.DockPoint = (Vector2)m_DockingPoints[formationIndex][followerIndex];
            m_Followers.Add(follower);
            return true;
        }

        public bool PushFollower(FollowerShip newFollower)
        {
            List<FollowerShip> tempList = new List<FollowerShip>(DOCKING_FORMATION_COUNT);
            foreach (FollowerShip follower in m_Followers)
            {
                tempList.Add(follower);
            }
            m_Followers.Clear();
            tempList.Add(newFollower);

            bool bAddedFollower = true;
            int i = 0;
            foreach (FollowerShip follower in tempList)
            {
                bAddedFollower = AddFollower(i, tempList.Count, follower);
                ++i;
                if (!bAddedFollower)
                    break;
            }
            return bAddedFollower;
        }

        public FollowerShip PopFollower()
        {
            if (m_Followers.Count > 0)
            {
                FollowerShip oldFollower = m_Followers[m_Followers.Count - 1];
                m_Followers.Remove(oldFollower);

                List<FollowerShip> tempList = new List<FollowerShip>(DOCKING_FORMATION_COUNT);
                foreach (FollowerShip follower in m_Followers)
                {
                    tempList.Add(follower);
                }
                m_Followers.Clear();

                int i = 0;
                foreach (FollowerShip follower in tempList)
                {
                    AddFollower(i, tempList.Count, follower);
                    ++i;
                }

                return oldFollower;
            }

            return null;
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            /*const float MOVEMENT_RATE = 150.0f; 

            KeyboardState currentState = Keyboard.GetState();
            Vector2 currentPos = this.Position;

            if( currentState.IsKeyDown(Keys.Up) )
                currentPos.Y += MOVEMENT_RATE;
            else if( currentState.IsKeyDown(Keys.Down) )
                currentPos.Y -= MOVEMENT_RATE;
            if (currentState.IsKeyDown(Keys.Right))
                currentPos.X += MOVEMENT_RATE;
            else if (currentState.IsKeyDown(Keys.Left))
                currentPos.X -= MOVEMENT_RATE;

            this.Position = currentPos;*/

            //const float MOVEMENT_RATE = 500.0f;
            float MOVEMENT_RATE = m_MaxForce * 1.0f;
            //float MOVEMENT_RATE = m_MaxSpeed *1.0f;

            KeyboardState currentState = Keyboard.GetState();
            Vector2 steeringForce = Vector2.Zero;

            if (currentState.IsKeyDown(Keys.Up))
                steeringForce.Y = MOVEMENT_RATE;
            else if (currentState.IsKeyDown(Keys.Down))
                steeringForce.Y = -MOVEMENT_RATE;
            
            if (currentState.IsKeyDown(Keys.Right))
                steeringForce.X = MOVEMENT_RATE;
            else if (currentState.IsKeyDown(Keys.Left))
                steeringForce.X = -MOVEMENT_RATE;

            steeringForce = Truncate(steeringForce, MOVEMENT_RATE);

            base.ApplySteeringForce(steeringForce);

            DebugText.Output("Player [{0}]", m_Position );
            
            base.Update(gameTime);
        }
    }
}


