#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using FormationGamePrototype.Graphics;
using FormationGamePrototype.Helper;
#endregion

namespace FormationGamePrototype.World
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class SteeringObject : Microsoft.Xna.Framework.GameComponent
    {
        protected SimpleModel m_Model;
        
        protected float m_Mass;
        protected Vector2 m_Position;
        protected Vector2 m_Velocity;
        protected float m_MaxForce;
        protected float m_MaxSpeed;
        protected float m_Direction;

        protected float m_Drag;
        
        public SteeringObject(Game game)
            : base(game)
        {
            m_Model = new SimpleModel(game);
            game.Components.Add(m_Model);

            m_Mass = 10.0f;
            m_Position = Vector2.Zero;
            m_Velocity = Vector2.Zero;
            m_MaxForce = 200.0f;
            m_MaxSpeed = 100.0f;
            m_Direction = 0.0f;
            
            m_Drag = 0.25f;
        }

        public void Destroy()
        {
            Game.Components.Remove(m_Model);
        }

        public SimpleModel Model
        {
            get
            {
                return m_Model;
            }
        }

        public Vector2 Position
        {
            set
            {
                m_Position = value;
            }
            get
            {
                return m_Position;
            }
        }

        public Vector2 Velocity
        {
            get
            {
                return m_Velocity;
            }
        }

        protected Vector2 Truncate( Vector2 inVec, float maxVal )
        {
            Vector2 ret = inVec;
            if (ret.Length() > maxVal)
            {
                ret.Normalize();
                ret *= maxVal;
            }
            return ret;
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }

        public void ApplySteeringForce(Vector2 steeringVector)
        {
            Vector2 steeringForce = steeringVector;
            steeringForce = Truncate(steeringForce, m_MaxForce);
            
            Vector2 acceleration = steeringForce / m_Mass;
            acceleration = Truncate(acceleration, m_MaxSpeed);
            
            m_Velocity += acceleration;

            DebugLine.Draw(m_Position, (m_Position + steeringForce * 10.0f), Color.Green);
            DebugLine.Draw(m_Position, (m_Position + m_Velocity * 10.0f), Color.Red);
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            m_Position += m_Velocity;
            m_Model.Position = new Vector3(m_Position, 0.0f);

            m_Velocity -= 0.5f * m_Velocity * m_Drag;
            base.Update(gameTime);
        }
    }
}


